
from torchannels import channels
import re,time

class BaseChannels( object ):
    """Mix-in providing the channel logic"""
    # We might also want '/' or even multilingual support?
    CHANNEL_NAME_MATCHER = re.compile( r'^([-a-zA-Z0-9]+)$')
    # a value that should never be able to show up in (valid) json...
    MESSAGE_SPLIT_TEXT = '""""""""""""""""""""""""""""'

    channel_class = channels.Channel

    @classmethod
    def valid_channel_name( cls, name ):
        """Is this a valid channel name?"""
        return bool(cls.CHANNEL_NAME_MATCHER.match( name ))
    def can_access( self, sub ):
        """Override to provide authenication for channels"""
        if not self.valid_channel_name( sub ):
            return False
        return True
    @classmethod
    def get_channel( cls, channel_name, existing_only=False ):
        """Retrieve a particular channel by name"""
        if existing_only:
            return cls.channel_class.get_existing( channel_name )
        else:
            return cls.channel_class.get_channel( channel_name )
    def subscribe( self, subscriptions, callback=None ):
        """Subscribe for updates to the given subscriptions"""
        for sub in subscriptions:
            self.get_channel( sub ).add_watcher( callback or self )
        return
    @property
    def invalid( self ):
        """Are we already marked invalid (closed or finished)?"""
        raise NotImplemented

    def updates( self, subscriptions, timestamp ):
        """Write all updates for given subscriptions after timestamp

        return bool() whether there were pending updates to write...
        """
        # alrighty, load these channels and see what's pending...
        pending = []
        for sub in subscriptions:
            ts = timestamp
            channel = self.get_channel( sub, existing_only=True )
            if channel:
                if not timestamp:
                    # see if we have an initial-state value...
                    state = channel.get_state()
                    if state:
                        ts = state['ts']
                        pending.extend( state['messages'] )
                pending.extend( channel.since( ts ))
        return pending
