#! /usr/bin/env python
"""Example application for serving TurboGears under Twisted

Twisted 9.x can host a WSGI application using a ThreadPool so that
each WSGI thread can block on e.g. DB operations.  You need to be
sure that your TurboGears application doesn't do any funky caching
within the Python namespace, but you shouldn't do that anyway, as
it would cause problems with scaling out.

Based on [J.P. Calderone's Twisted WSGI in 60 Seconds http://jcalderone.livejournal.com/51888.html]

Usage:
    virtualenv --no-site-packages twisted-tg
    cd twisted-tg
    source bin/activate
    easy_install -U setuptools
    easy_install -i http://www.turbogears.org/2.1/downloads/current/index tg.devtools
    easy_install twisted

    paster quickstart myproject
    cd myproject
    python setup.py develop
    paster setup-app development.ini
    python ../../twistedtg.py development.ini

which should serve the new application in dev.ini on port 8080.

Tested with Python 2.6, Twisted 9.0 and TurboGears 2.1 on Ubuntu Karmic.
"""
from twisted.web.server import Site
from twisted.web.resource import Resource
from twisted.internet import reactor
from twisted.web.server import NOT_DONE_YET
import os

class Root( Resource ):
    isLeaf=True
    def render_GET( self, request ):
        return open( 'hellostream.html' ).read()
    def render_POST( self, request ):
        request.setHeader('Content-Type','text/plain')
        reactor.callLater( 1, self.stream_data, request )
        return NOT_DONE_YET
    def stream_data( self, request ):
        """Stream some text out to the request"""
        try:
            request.write( 'Hello ' )
        except Exception, err:
            print 'Stopping streaming', err
        else:
            reactor.callLater( 1, self.stream_data, request )

def main(port=8080):
    resource = Root()
    factory = Site(resource)
    reactor.listenTCP( int(port), factory )
    reactor.run()

if __name__ == "__main__":
    import sys
    main( *sys.argv[1:] )
